#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
#include "tm5.inc"

! Quick access to a single variable from an HDF or netCDF file, via MDF interface.

module ReadFromFile_MDF
  
  use GO,		only: gol, goErr, goPr, goLabel
  use MDF

  implicit none

  character(len=*), parameter	:: mname = 'ReadFromFile_MDF'

  public

  ! This interface is not complete.  If you need a differently-typed or -shaped
  ! variable read function, just add it.
  interface ReadFromFile
     module procedure ReadFromFile_r4_1d
     module procedure ReadFromFile_r4_2d
     module procedure ReadFromFile_r4_3d
     module procedure ReadFromFile_r4_4d
     module procedure ReadFromFile_r8_1d
     module procedure ReadFromFile_r8_2d
     module procedure ReadFromFile_r8_3d
     module procedure ReadFromFile_r8_4d
     module procedure ReadFromFile_i2_1d
     module procedure ReadFromFile_i2_2d
     module procedure ReadFromFile_i2_3d
     module procedure ReadFromFile_i2_4d
     module procedure ReadFromFile_i4_1d
     module procedure ReadFromFile_i4_2d
     module procedure ReadFromFile_i4_3d
     module procedure ReadFromFile_i4_4d
     module procedure ReadFromFile_c1_1d
     module procedure ReadFromFile_c1_2d
     module procedure ReadFromFile_c1_3d
     module procedure ReadFromFile_c1_4d
  end interface

contains

  subroutine ReadFromFile_r4_1d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(4), intent(out)		:: buffer(:)
    integer, intent(out)		:: status
    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_r4_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_r4_1d

  subroutine ReadFromFile_r4_2d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(4), intent(out)		:: buffer(:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_r4_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_r4_2d

  subroutine ReadFromFile_r4_3d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(4), intent(out)		:: buffer(:,:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_r4_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_r4_3d

  subroutine ReadFromFile_r4_4d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(4), intent(out)		:: buffer(:,:,:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_r4_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_r4_4d

  subroutine ReadFromFile_r8_1d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(8), intent(out)		:: buffer(:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_r8_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_r8_1d

  subroutine ReadFromFile_r8_2d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(8), intent(out)		:: buffer(:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_r8_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_r8_2d

  subroutine ReadFromFile_r8_3d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(8), intent(out)		:: buffer(:,:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_r8_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_r8_3d

  subroutine ReadFromFile_r8_4d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    real(8), intent(out)		:: buffer(:,:,:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_r8_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_r8_4d

  subroutine ReadFromFile_i2_1d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(2), intent(out)		:: buffer(:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_i2_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_i2_1d

  subroutine ReadFromFile_i2_2d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(2), intent(out)		:: buffer(:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_i2_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_i2_2d

  subroutine ReadFromFile_i2_3d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(2), intent(out)		:: buffer(:,:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_i2_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_i2_3d

  subroutine ReadFromFile_i2_4d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(2), intent(out)		:: buffer(:,:,:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_i2_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_i2_4d

  subroutine ReadFromFile_i4_1d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(4), intent(out)		:: buffer(:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_i4_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_i4_1d

  subroutine ReadFromFile_i4_2d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(4), intent(out)		:: buffer(:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_i4_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_i4_2d

  subroutine ReadFromFile_i4_3d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(4), intent(out)		:: buffer(:,:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_i4_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_i4_3d

  subroutine ReadFromFile_i4_4d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    integer(4), intent(out)		:: buffer(:,:,:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_i4_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_i4_4d

  subroutine ReadFromFile_c1_1d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    character, intent(out)		:: buffer
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_c1_1d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_c1_1d

  subroutine ReadFromFile_c1_2d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    character, intent(out)		:: buffer(:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_c1_2d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_c1_2d

  subroutine ReadFromFile_c1_3d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    character, intent(out)		:: buffer(:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_c1_3d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_c1_3d

  subroutine ReadFromFile_c1_4d(fname,ftype,vname,buffer,status)

    character(len=*), intent(in)	:: fname  ! file name
    integer, intent(in)			:: ftype  ! MDF_HDF4, MDF_NETCDF, MDF_NETCDF4, MDF_HDF5
    character(len=*), intent(in)	:: vname  ! variable name
    character, intent(out)		:: buffer(:,:,:)
    integer, intent(out)		:: status

    integer				:: hid
    integer				:: vid

    character(len=*), parameter		:: rname = mname//'/ReadFromFile_c1_4d'

    call MDF_open(fname,ftype,mode=MDF_READ,hid=hid,status=status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Inq_VarID(hid,vname,vid,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Get_Var(hid,vid,buffer,status)
    IF_NOTOK_RETURN(status=1)

    call MDF_Close(hid,status)
    IF_NOTOK_RETURN(status=1)

    status = 0

  end subroutine ReadFromFile_c1_4d

end module ReadFromFile_MDF
